/*
Esempio NodeMCU con sensore di temperatura digitale DS18S20 e trasmissione
stato via MQTT verso NodeRed.

Questo esempio fa parte della guida di HTML.it "Domotica per Makers"
consultabile all'indirizzo:
https://www.html.it/guide/domotica-per-makers/

L'esempio è realizzato da Giovanni Bernardo (https://www.twitter.com/settorezero)

PREREQUISITI:
- Libreria Dallas Temperature by Miles Burton
- Libreria PubSubClient by Nick O'Leary

*/
 
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <OneWire.h>
#include <DallasTemperature.h>


// *** INIZIO SETUP

// IMPOSTAZIONI RETE WI-FI
const char* ssid = "SSID"; // nome della rete Wi-Fi
const char* password = "PASSWORD"; // password della rete Wi-Fi
IPAddress ip(192,168,1,133); // indirizzo IP da assegnare a questo dispositivo
IPAddress gateway(192,168,1,1); // indirizzo IP del tuo router/access-point Wi-Fi
IPAddress subnet(255,255,255,0); // subnet mask

// IMPOSTAZIONI PER LA COMUNICAZIONE MQTT
const char* mqtt_server = "192.168.1.101"; // Indirizzo IP del tuo server MQTT
const char* mqtt_username = "MQTT_USERNAME"; // Nome utente da utilizzare per l'accesso al broker MQTT (metti "" se non hai impostato utente/password)
const char* mqtt_password = "MQTT_PASSWORD"; // Password del tuo server MQTT (metti "" se non hai impostato utente/password)
const uint16_t mqtt_port = 1883; // Porta del tuo server MQTT (default: 1883)
const char* mqtt_client_id = "NodeMCU_Temperatura_digitale"; // Nome univoco a piacere da assegnare a questo dispositivo (due dispositivi sulla rete non possono avere lo stesso nome!)
const char* mqtt_topic = "esempio/temperatura"; // Topic sul quale pubblicare il messaggio

#define ONE_WIRE_BUS 5 // terminale data Out del sensore di temperatura sul GPIO 5 (D1)

// *** FINE SETUP

WiFiClient espClient;
PubSubClient client(mqtt_server, mqtt_port, espClient);
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

// sul GPIO 2 (D4) del nodeMCU c'è il led blu
#define LED         2 // (D4)

void mqtt_connect(void) 
  {
  // Eseguiamo un loop continuo fino a che non ci colleghiamo al broker MQTT
  while (!client.connected()) 
    {
    Serial.println("Provo a collegarmi al broker MQTT");
    if (client.connect(mqtt_client_id, mqtt_username, mqtt_password)) 
        {
        Serial.println("Connesso!");
        } 
    else 
        {
        Serial.print("Connessione fallita, codice=");
        Serial.print(client.state());
        Serial.println(" - riprovo tra 5 secondi");
        delay(5000);
        }
    }
  }

void setup() 
  {
  // inizializzo il GPIO dove c'è collegato il led
  pinMode(LED,OUTPUT); // faccio in modo che il GPIO 2, al quale è collegato il led, sia un'uscita
  digitalWrite(LED,HIGH); // led spento. Sul NodeMCU Lolin il led si spegne inviando un livello alto

  // inizializzo la porta seriale, che userò come debug
  Serial.begin(115200); // baudrate a 115200bps

  // inizializzo la libreria per la lettura dei sensori Dallas
  sensors.begin();
  
  // inizializzo il Wi-Fi
  Serial.print("Mi sto collegando alla rete Wi-Fi ");
  Serial.println(ssid);

  // Imposto gli IP definiti nella configurazione
  WiFi.config(ip,gateway,subnet);
  // avvio la connessione
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) 
    {
    delay(500);
    Serial.print(".");
    }
  
  Serial.println("");
  Serial.println("Sono connesso al Wi-Fi!");
  digitalWrite(LED,LOW); // accendo il led per segnalare che mi sono collegato

  // Mi collego al broker MQTT
  mqtt_connect();
  }

bool check_temp(void)
  {
  bool published=false;
  
  sensors.requestTemperatures(); // Invio il comando per richiedere la temperatura

  Serial.print("Temperatura: ");
  float temp=sensors.getTempCByIndex(0); // ottengo la temperatura dal 1° (e unico, nel nostro esempio) sensore sul bus
  Serial.println(temp); 

  // dato che il metodo publish accetta come payload solo una valore in formato char, dobbiamo 
  // convertire il valore di temperatura, che è una variabile float, in un array
  // utilizzando la funzione di Arduino dtostrf
  char tempChar[7]; // buffer per contenere il valore di temperatura espresso come array
  dtostrf(temp, 4, 1, tempChar);  // 5 è la larghezza minima della stringa, 1 è il numero di decimali dopo la virgola
  
  published=client.publish(mqtt_topic,tempChar);
  return (published);
  }

void loop() 
  {
  static long lastMsg=millis();

  // ad ogni giro controllo che sono connesso al broker MQTT, in caso contrario
  // rieseguo la connessione
  if (!client.connected()) 
      {
      mqtt_connect();
      }
  client.loop();

  // ogni 5 secondi controllo la temperatura
  if (millis() - lastMsg > 5000) 
    {
    lastMsg = millis();
    if (check_temp())
      {
      Serial.println("Messaggio pubblicato!");
      }
      else
      {
       Serial.println("Errore pubblicazione Messaggio"); 
      }
    }
  }
  
